//////////////////////////////////////////////////////
//  Importer
//////////////////////////////////////////////////////

import java.awt.*;

//////////////////////////////////////////////////////
//  Klass Block
//////////////////////////////////////////////////////

/**  Ett Tetrisblock. */
public class Block
{
    /** Flytta blocket t vnster. */
    public static final int LEFT=0;
    /** Flytta blocket t hger. */
    public static final int RIGHT=1;
    /** Flytta ner blocket. */
    public static final int DOWN=2;
    /** Rotera blocket medurs. */
    public static final int CLOCKWISE=3;
    /** Rotera blocket moturs. */
    public static final int COUNTERCLOCKWISE=4;
    
    private int block, x, y, angle;
    
    // Data fr att stta ut block i next-rutan
    private int xOffset[]={ 8,  0,  8,  8,  8, 16,  0},
	yOffset[]={ 0,  0, 16, 16, 16,  8,  8};
    
    // Ren och skr data fr blockens utseende.
    private final boolean b[][][]={
	{{false,  true, false, false, false,  true, false, false, false,  true, false, false, false,  true, false, false}, {false, false, false, false, false, false, false, false,  true,  true,  true,  true, false, false, false, false}, {false,  true, false, false, false,  true, false, false, false,  true, false, false, false,  true, false, false}, {false, false, false, false, false, false, false, false,  true,  true,  true,  true, false, false, false, false}},
	{{false, false, false, false, false,  true,  true, false, false,  true,  true, false, false, false, false, false}, {false, false, false, false, false,  true,  true, false, false,  true,  true, false, false, false, false, false}, {false, false, false, false, false,  true,  true, false, false,  true,  true, false, false, false, false, false}, {false, false, false, false, false,  true,  true, false, false,  true,  true, false, false, false, false, false}},
	{{false, false, false, false, false, false, false, false,  true,  true,  true, false, false,  true, false, false}, {false, false, false, false, false,  true, false, false,  true,  true, false, false, false,  true, false, false}, {false, false, false, false, false,  true, false, false,  true,  true,  true, false, false, false, false, false}, {false, false, false, false, false,  true, false, false, false,  true,  true, false, false,  true, false, false}},
	{{false, false, false, false, false, false, false, false, false,  true,  true, false,  true,  true, false, false}, {false, false, false, false,  true, false, false, false,  true,  true, false, false, false,  true, false, false}, {false, false, false, false, false, false, false, false, false,  true,  true, false,  true,  true, false, false}, {false, false, false, false,  true, false, false, false,  true,  true, false, false, false,  true, false, false}},
	{{false, false, false, false, false, false, false, false,  true,  true, false, false, false,  true,  true, false}, {false, false, false, false, false,  true, false, false,  true,  true, false, false,  true, false, false, false}, {false, false, false, false, false, false, false, false,  true,  true, false, false, false,  true,  true, false}, {false, false, false, false, false,  true, false, false,  true,  true, false, false,  true, false, false, false}},
	{{false, false, false, false, false,  true, false, false, false,  true, false, false,  true,  true, false, false}, {false, false, false, false,  true, false, false, false,  true,  true,  true, false, false, false, false, false}, {false, false, false, false, false,  true,  true, false, false,  true, false, false, false,  true, false, false}, {false, false, false, false, false, false, false, false,  true,  true,  true, false, false, false,  true, false}},
	{{false, false, false, false, false,  true, false, false, false,  true, false, false, false,  true,  true, false}, {false, false, false, false, false, false, false, false,  true,  true,  true, false,  true, false, false, false}, {false, false, false, false,  true,  true, false, false, false,  true, false, false, false,  true, false, false}, {false, false, false, false, false, false,  true, false,  true,  true,  true, false, false, false, false, false}}};
    
    /** Konstruerar Tetrisblock nr n, n=0...6.
	Om n r mindre n noll eller strre n 6 justeras det drefter. */
    public Block(int n)
    {
	if(n<0) n=-n;
	block=n%7;
	x=4;
	y=-1;
	angle=0;
    }
    
    /** Ritar ett block i "Next-rutan". */
    public void showAsNext(Graphics g, Box box)
    {
	for(int j=0;j<4;j++)
	    for(int i=0;i<4;i++)
		if(b[block][0][i+j*4])
		    box.plotExact(g, block, i*16+xOffset[block], j*16-yOffset[block]);
    }
    
    /** Frsker flytta p blocket i spelfltet. */
    public boolean move(Playfield f, int whereto)
    {
	switch(whereto)
	    {
	    case LEFT:
		remove(f);
		if(canPut(f, x-1, y, angle))
		    {
			x--;
			put(f);
			return true;
		    }
		else
		    {
			put(f);
			return false;
		    }
	    case RIGHT:
		remove(f);
		if(canPut(f, x+1, y, angle))
		    {
			x++;
			put(f);
			return true;
		    }
		else
		    {
			put(f);
			return false;
		    }
	    case DOWN:
		remove(f);
		if(canPut(f, x, y+1, angle))
		    {
			y++;
			put(f);
			return true;
		    }
		else
		    {
			put(f);
			return false;
		    }
	    case CLOCKWISE:
		remove(f);
		if(canPut(f, x, y, (angle+1)&3))
		    {
			angle=(angle+1)&3;
			put(f);
			return true;
		    }
		else
		    {
			put(f);
			return false;
		    }
	    case COUNTERCLOCKWISE:
		remove(f);
		if(canPut(f, x, y, (angle-1)&3))
		    {
			angle=(angle-1)&3;
			put(f);
			return true;
		    }
		else
		    {
			put(f);
			return false;
		    }
	    }
	// Om du kom hit s skickade du inte med rtt saker...
	return false;
    }
    
    /** Kolla om man kan stta ut blocket. */
    public boolean canPut(Playfield f)
    {
	return canPut(f, x, y, angle);
    }
    
    /** Kolla om man kan stta ut blocket. */
    private boolean canPut(Playfield f, int x, int y, int a)
    {
	for(int j=0;j<4;j++)
	    for(int i=0;i<4;i++)
		if(b[block][a][i+j*4])
		    if(f.get(x+i, y+j)>=0)
			return false;
	return true;
    }
    
    /** Ritar ut blocket p spelfltet. */
    public void put(Playfield f)
    {
	for(int j=0;j<4;j++)
	    for(int i=0;i<4;i++)
		if(b[block][angle][i+j*4])
		    f.put(block, x+i, y+j);
    }
    
    /** Suddar ut blocket frn spelfltet. */
    public void remove(Playfield f)
    {
	for(int j=0;j<4;j++)
	    for(int i=0;i<4;i++)
		if(b[block][angle][i+j*4])
		    f.put(-1, x+i, y+j);
    }
    
    /** Returnerar blockets position i x-led. */
    public int getX()
    {
	return x;
    }
    
    /** Returnerar blockets position i y-led. */
    public int getY()
    {
	return y;
    }
}
