//////////////////////////////////////////////////////
//  Importer
//////////////////////////////////////////////////////

import java.net.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.lang.*;

//////////////////////////////////////////////////////
//  Huvudklass Tetris
//////////////////////////////////////////////////////

/** Originally created by Alexey Pazhitnov.<BR>
    This java version was made by David hlin. */
public class Tetris
    extends Applet
    implements KeyListener, MouseListener, Runnable, ScoreReceiver
{
    private Thread thread;        // Fr att blocken ska hoppa ner av sig sjlva
    
    private Image bakgrund;       // Appletbakgrund
    private Image btnStart;       // Start-knapp
    private Image btnDead;        // Dd-knapp
    // Mer bilder:
    private Image plus_on, plus_off, minus_on, minus_off;
    private Image [] ha, hb;
    
    private int pfw, pfh;         // Bredd och hjd p spelplanen
    private int scx, scy;         // Placering av pongen p skrmen
    
    private Block current, next;  // Block som anvnds
    private Box b;                // Ldor fr grafik
    private Playfield f;          // Spelfltet
    
    private Alpha abc;            // Objekt fr text- och sifferutskrift via det
                                  // interna systemet med bilder
    
    private int lines;            // Antal rader som har tagits bort
    private int score;            // Uppndd pong.
    private int startH;           // Den hjd som man startar p.
    private int startL;           // Den niv som man startar p.
    
    private int screenScore;      // Pong som syns p skrmen (animeras).
    private ScoreAnimator sa;     // Pongobjekt, som animerar pongen p skrmen
    
    private int currentMode;      // Aktuellt appletlge
    private final int GAME=0, START=1, DEAD=2;
    
    // Koordinater fr klickbara ytor
    private final int hMinusX=56, hMinusY=49, hMinusW=48, hMinusH=48,
	hPlusX=120, hPlusY=48, hPlusW=48, hPlusH=48;
    private final int lMinusX=216, lMinusY=53, lMinusW=48, lMinusH=48,
	lPlusX=392, lPlusY=53, lPlusW=48, lPlusH=48;
    private final int startButtonX=224, startButtonY=160,
	startButtonW=64, startButtonH=64;
    private final int deadButtonX=224, deadButtonY=160,
	deadButtonW=64, deadButtonH=64;
    
    /** Mlar om skrmen */
    public void paint(Graphics g)
    {
	switch(currentMode)
	    {
	    case START:
				// Bakgrund
		g.drawImage(bakgrund, 0, 0, this);
				// Start-knapp
		g.drawImage(btnStart, startButtonX, startButtonY, this);
				// Plus/minus starthjd
		if(startH>0) g.drawImage(minus_on, hMinusX, hMinusY, this);
		else         g.drawImage(minus_off, hMinusX, hMinusY, this);
		if(startH<5) g.drawImage(plus_on, hPlusX, hPlusY, this);
		else         g.drawImage(plus_off, hPlusX, hPlusY, this);
				// Starthjd
		for(int i=0;i<5;i++)
		    if(i<startH) g.drawImage(ha[i], 32, 256-32*i, this);
		    else           g.drawImage(hb[i], 32, 256-32*i, this);
				// Plus/minus startniv
		if(startL>0) g.drawImage(minus_on, lMinusX, hMinusY, this);
		else         g.drawImage(minus_off, lMinusX, hMinusY, this);
		if(startL<9) g.drawImage(plus_on, lPlusX, hPlusY, this);
		else         g.drawImage(plus_off, lPlusX, hPlusY, this);
				// Startniv
		abc.print(g, "NIV: "+intToString(startL, 1), scx+4, scy+1);
				// Info
		abc.print(g, "KLICKA I", 19, 7);
		abc.print(g, "RUTAN FR", 19, 8);
		abc.print(g, "ATT STARTA", 19, 9);
		abc.print(g, "TETRIS", 19, 10);
		break;
	    case DEAD:
				// Bakgrund
		g.drawImage(bakgrund, 0, 0, this);
				// Spelfltet
		b.plot(g.create(32, 32, 16*pfw, 16*pfh), f);
				// Text, data
		abc.print(g, "PONG: "+intToString(screenScore, 7), scx, scy);
		abc.print(g, "RADER:     "+intToString(lines, 3), scx, scy+1);
		abc.print(g, "NIV:       "+intToString(getLevel(), 2), scx, scy+2);
				// Spindel-knapp
		g.drawImage(btnDead, deadButtonX, deadButtonY, this);
				// Info
		abc.print(g, "DU DOG...", 19, 7);
		abc.print(g, "TRYCK HR", 19, 8);
		abc.print(g, "FR ATT", 19, 9);
		abc.print(g, "BRJA OM", 19, 10);
		break;
	    case GAME:
				// Bakgrund
		g.drawImage(bakgrund, 0, 0, this);
				// Spelfltet
		b.plot(g.create(32, 32, 16*pfw, 16*pfh), f);
				// Nextrutan
		next.showAsNext(g.create(224, 160, 64, 64), b);
				// Text, data
		abc.print(g, "PONG: "+intToString(screenScore, 7), scx, scy);
		abc.print(g, "RADER:     "+intToString(lines, 3), scx, scy+1);
		abc.print(g, "NIV:       "+intToString(getLevel(), 2), scx, scy+2);
		break;
	    }
    }
    
    /** Initierar spelet och de objekt som ska anvndas. */
    public void init()
    {
	// Stt variabler
	currentMode=START;
	pfw=10; pfh=16;
	scx=13; scy=2;
	b=new Box(this);
	abc=new Alpha(this);
	sa=new ScoreAnimator(this, 35, 11);
	startH=0;
	
	// Gr flimret lite behagligare
	setBackground(Color.black);
	
	// Ladda in grafik
	MediaTracker tracker=new MediaTracker(this);
	bakgrund=  getImage(getCodeBase(), "pix/background.jpg");
	btnStart=  getImage(getCodeBase(), "pix/btnstart.jpg");
	btnDead=   getImage(getCodeBase(), "pix/btndead.jpg");
	plus_on=   getImage(getCodeBase(), "pix/plus.jpg");
	plus_off=  getImage(getCodeBase(), "pix/plusx.jpg");
	minus_on=  getImage(getCodeBase(), "pix/minus.jpg");
	minus_off= getImage(getCodeBase(), "pix/minusx.jpg");
	ha=new Image [5];
	hb=new Image [5];
	for(int i=0;i<5;i++)
	    ha[i]=getImage(getCodeBase(), "pix/h"+(i+1)+"a.jpg");
	for(int i=0;i<5;i++)
	    hb[i]=getImage(getCodeBase(), "pix/h"+(i+1)+"b.jpg");
	// Vnta p att grafiken laddas klart
	int n=0;
	tracker.addImage(bakgrund,  n++);
	tracker.addImage(btnStart,  n++);
	tracker.addImage(btnDead,   n++);
	tracker.addImage(plus_on,   n++);
	tracker.addImage(plus_off,  n++);
	tracker.addImage(minus_on,  n++);
	tracker.addImage(minus_off, n++);
	for(int i=0;i<5;i++)
	    tracker.addImage(ha[i], n++);
	for(int i=0;i<5;i++)
	    tracker.addImage(hb[i], n++);
	
	try{tracker.waitForAll();}
	catch(InterruptedException e){}
	
	// Lyssna p spelaren nr han/hon slr febrilt p tangentbordet
	// eller misshandlar musen
	addKeyListener(this);
	addMouseListener(this);
    }
    
    /** Startar igng Tetris. */
    private void startGame()
    {
	f=new Playfield(pfw, pfh, startH);
	next=new Block(randomNR());
	newBlock();
	current.put(f);
	lines=0;
	score=0;
	screenScore=0;
	runGame();
	repaint();
    }
    
    /** Startar appleten. */
    public void start()
    {
	switch(currentMode)
	    {
	    case GAME:
		runGame();
		break;
	    }
    }
    
    /** Stter spelet i rullning. */
    private void runGame()
    {
	thread=new Thread(this);
	thread.start();
	sa.start();
    }
    
    /** Stoppar spelet. */
    private void stopGame()
    {
	thread=null;
	sa.stop();
    }
    
    /** Stoppar/pausar appleten. */
    public void stop()  // Men den hr anropas ju aldrig...!
    {
	switch(currentMode)
	    {
	    case GAME:
		stopGame();
		break;
	    }
    }
    
    /** Skapar ett nytt Tetrisblock. */
    private void newBlock()
    {
	current=next;
	next=new Block(randomNR());
	if(!next.canPut(f))
	    {
		stopGame();
		currentMode=DEAD;
		repaint();
	    }
	current.put(f);
	repaint(224, 160, 16*4, 16*4);
    }
    
    /** Vntar en stund, och flyttar sedan ner blocket ett snpp.
	Om detta redan har skett sedan run() anropades s sker inget. */
    public void run()
    {
	if(currentMode==GAME)
	    {
		Thread thisThread=Thread.currentThread();
		try{thisThread.sleep(getDelay());}
		catch(InterruptedException e){}
		if(thisThread==thread)
		    {
			getDown();
			repaintCurrent();
		    }
	    }
    }
    
    /** Anropas nr en tangent trycks ner. */
    public void keyPressed(KeyEvent e)
    {
	switch(currentMode)
	    {
	    case GAME:
		switch(e.getKeyCode())
		    {
		    case e.VK_UP:	// Rotera medurs
			current.move(f, current.CLOCKWISE);
			break;
		    case e.VK_LEFT:  // Flytta vnster
			current.move(f, current.LEFT);
			break;
		    case e.VK_RIGHT: // Flytta hger
			current.move(f, current.RIGHT);
			break;
		    case e.VK_DOWN:  // Hoppa ner en rad
			getDown();
			break;
		    default:
			return;
		    }
		repaintCurrent();
		break;
	    }
    }
    
    /** Anropas nr en tangent slpps. */
    public void keyReleased(KeyEvent e){}
    
    /** Anropas nr ett tecken har matats in. */
    public void keyTyped(KeyEvent e){}
    
    /** Mlar om skrmen, dock bara kring blocket.
	Detta fr att minska flimret. */
    private void repaintCurrent()
    {
	repaint(16+16*current.getX(), 16+16*current.getY(), 16*6, 16*5);
    }
    
    /** Returnerar delay-vrdet i millisekunder fr den aktuella spelnivn.
	Anvnds i Thread.sleep(). */
    private int getDelay()
    {
	return 4800/(getLevel()+4); // millisekunder
    }
    
    /** Returnerar nuvarande spelniv. */
    private int getLevel()
    {
	return (lines/10)+startL; // kar med ett var tionde rad man fr
    }
    
    /** Uppdaterar pongen map hur mnga rader som eliminerades,
	samt vilken niv spelaren har uppntt. */
    private void addScore(int rows)
    {
	final int sc[]={1, 5, 20, 50, 200};
	score+=(int)((double)sc[rows]*(getLevel()+2)*(startH+4.0)/4.0);
	sa.setScore(score);
    }
    
    /** Flyttar ner blocket en rad,
	och nollstller den automatiska nedflyttningen. */
    private void getDown()
    {
	thread=null;
	if(!current.move(f, current.DOWN))
	    {
		// Om det inte gr att flytta ner blocket
		// s har det gtt p grund.
		int n=f.checkLines();
		addScore(n);
		if(n>0)
		    {
				// Om rader har flyttats ner...
			lines+=n;
			repaint();
		    }
		newBlock();
	    }
	// Startar om den automatiska nedflyttningen.
	thread=new Thread(this);
	thread.start();
    }
    
    /** Hmtar ett slumptal av typen integer mellan 0 och 7, inklusive. */
    private int randomNR()
    {
	return (int)(Math.random()*8.0);
    }
    
    /** Det ryktas att den hr gr s att skrmen mlas om utan flimmer. */
    public void update(Graphics g)
    {
	paint(g);
    }
    
    /** Uppdaterar pongen. */
    public void printScore(int score)
    {
	screenScore=score;
	//repaint();
	repaint(abc.getWidth()*(scx+7), abc.getHeight()*scy, abc.getWidth()*9, abc.getHeight());
    }
    
    /** Konverterar i till en strng med n tecken. */
    private String intToString(int i, int n)
    {
	String s="";
	int k=0;
	do
	    {
		s=(i%10)+s;
		i/=10;
		k++;
	    } while(k<n && i!=0);
	for(;k<n;k++)
	    s=" "+s;
	
	return s;
    }
    
    /** Anropas nr musenknappen slppts upp. */
    public void mouseReleased(MouseEvent e)
    {
	int x=e.getX(), y=e.getY();
	
	switch(currentMode)
	    {
	    case START:
				// Start-knappen
		if(x>=startButtonX &&
		   x<(startButtonX+startButtonW) &&
		   y>=startButtonY &&
		   y<(startButtonY+startButtonH))
		    {
			startGame();
			currentMode=GAME;
			return;
		    }
		
				// Plus starthjd
		if(x>=hPlusX &&
		   x<(hPlusX+hPlusW) &&
		   y>=hPlusY &&
		   y<(hPlusY+hPlusH))
		    {
			if(startH<5)
			    {
				startH++;
				repaint();
			    }
			return;
		    }
		
				// Minus starthjd
		if(x>=hMinusX &&
		   x<(hMinusX+hMinusW) &&
		   y>=hMinusY &&
		   y<(hMinusY+hMinusH))
		    {
			if(startH>0)
			    {
				startH--;
				repaint();
			    }
			return;
		    }
		
				// Plus startniv
		if(x>=lPlusX &&
		   x<(lPlusX+lPlusW) &&
		   y>=lPlusY &&
		   y<(lPlusY+lPlusH))
		    {
			if(startL<9)
			    {
				startL++;
				repaint();
			    }
			return;
		    }
		
				// Minus startniv
		if(x>=lMinusX &&
		   x<(lMinusX+lMinusW) &&
		   y>=lMinusY &&
		   y<(lMinusY+lMinusH))
		    {
			if(startL>0)
			    {
				startL--;
				repaint();
			    }
			return;
		    }
		break;
	    case DEAD:
				// Spindel-knappen
		if(x>=deadButtonX &&
		   x<(deadButtonX+deadButtonW) &&
		   y>=deadButtonY &&
		   y<(deadButtonY+deadButtonH))
		    {
			currentMode=START;
			repaint();
			return;
		    }
		break;
	    }
    }
    
    /** Anropas nr muspekaren ntrar appleten. */
    public void mouseEntered(MouseEvent e){}
    
    /** Anropas nr muspekaren lmnar appleten. */
    public void mouseExited(MouseEvent e){}
    
    /** Anropas nr musknappen trycks ner. */
    public void mousePressed(MouseEvent e){}
    
    /** Anropas nr musknappen klickats. */
    public void mouseClicked(MouseEvent e){}
}
